'''
    Algorithms for Pillar. Currently includes 4 "mini" algorithms with different 
    options, nothing too sophisticated.
HOW TO
    from pillaralgos import OPTIONS  # replace options with an algo
    OPTIONS.run(data)                # see algo docstring for descriptions
'''
__version__ = '1.0.6'